﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace LibraryManagement
{
    public class Catalog
    {
        // Liste aller Bücher im Katalog
        private List<Book> books;

        // Konstruktor
        public Catalog()
        {
            books = new List<Book>();
        }

        // Methode zum Hinzufügen eines Buches
        public void AddBook(Book book)
        {
            books.Add(book);
            Console.WriteLine($"'{book.Title}' wurde zum Katalog hinzugefügt.");
        }

        // Methode zum Entfernen eines Buches nach ISBN
        public void RemoveBook(string isbn)
        {
            Book? book = FindBookByISBN(isbn);
            if (book != null)
            {
                books.Remove(book);
                Console.WriteLine($"'{book.Title}' wurde aus dem Katalog entfernt.");
            }
            else
            {
                Console.WriteLine($"Buch mit ISBN {isbn} nicht gefunden.");
            }
        }

        // Methode zur Suche eines Buches nach ISBN
        public Book? FindBookByISBN(string isbn)
        {
            return books.FirstOrDefault(b => b.ISBN == isbn);
        }

        // Methode zur Suche von Büchern nach Titel und/oder Autor
        public List<Book> SearchBooks(string? title = null, string? author = null)
        {
            IEnumerable<Book> results = books;

            if (!string.IsNullOrEmpty(title))
            {
                results = results.Where(b => b.Title.IndexOf(title, StringComparison.OrdinalIgnoreCase) >= 0);
            }

            if (!string.IsNullOrEmpty(author))
            {
                results = results.Where(b => b.Author.IndexOf(author, StringComparison.OrdinalIgnoreCase) >= 0);
            }

            return results.ToList();
        }

        // Methode zur Auflistung aller Bücher (optional)
        public void ListAllBooks()
        {
            Console.WriteLine("Alle Bücher im Katalog:");
            foreach (var book in books)
            {
                Console.WriteLine($"- {book.Title} von {book.Author} (ISBN: {book.ISBN})");
            }
        }
    }
}

