﻿using System;
using System.Collections.Generic;
using System.Linq; // Erleichtert die Suche und Filterung von Listen

namespace LibraryManagement
{
    public class Library
    {
        // Eigenschaften
        private Catalog catalog;
        private List<Member> members;
        private List<Loan> loans;

        // Konstruktor
        public Library()
        {
            catalog = new Catalog();
            members = new List<Member>();
            loans = new List<Loan>();
        }

        // Methode zum Hinzufügen eines Buches zum Katalog
        public void AddBookToCatalog(Book book)
        {
            catalog.AddBook(book);
        }

        // Methode zum Entfernen eines Buches aus dem Katalog
        public void RemoveBookFromCatalog(string isbn)
        {
            catalog.RemoveBook(isbn);
        }

        // Methode zur Registrierung eines Mitglieds
        public void RegisterMember(Member member)
        {
            members.Add(member);
            Console.WriteLine($"Mitglied '{member.Name}' wurde registriert.");
        }

        // Methode zum Ausleihen eines Buches
        public void LoanBook(string isbn, string memberId)
        {
            Book? book = catalog.FindBookByISBN(isbn);
            Member? member = FindMemberById(memberId);

            if (book != null && member != null)
            {
                if (book.IsAvailable)
                {
                    member.BorrowBook(book);
                    Loan loan = new Loan(book, member);
                    loans.Add(loan);
                    Console.WriteLine($"Ausleihe registriert: {member.Name} - '{book.Title}' (Fällig am: {loan.DueDate.ToShortDateString()})");
                }
                else
                {
                    Console.WriteLine($"'{book.Title}' ist derzeit nicht verfügbar.");
                }
            }
            else
            {
                Console.WriteLine("Buch oder Mitglied nicht gefunden.");
            }
        }

        // Methode zur Rückgabe eines Buches
        public void ReturnBook(string isbn, string memberId)
        {
            Book? book = catalog.FindBookByISBN(isbn);
            Member? member = FindMemberById(memberId);

            if (book != null && member != null)
            {
                member.ReturnBook(book);
                Loan? loan = FindLoan(book, member);
                if (loan != null)
                {
                    loans.Remove(loan);
                    Console.WriteLine($"Ausleihe entfernt: {member.Name} - '{book.Title}'");
                }
                else
                {
                    Console.WriteLine("Keine entsprechende Ausleihe gefunden.");
                }
            }
            else
            {
                Console.WriteLine("Buch oder Mitglied nicht gefunden.");
            }
        }

        // Methode zur Suche eines Mitglieds nach ID
        private Member? FindMemberById(string memberId)
        {
            return members.FirstOrDefault(m => m.MemberID == memberId);
        }

        // Methode zur Suche einer Ausleihe
        private Loan? FindLoan(Book book, Member member)
        {
            return loans.FirstOrDefault(l => l.Book == book && l.Member == member);
        }

        // Methode zur Suche von Büchern
        public List<Book> SearchBooks(string? title = null, string? author = null)
        {
            return catalog.SearchBooks(title, author);
        }

        // Optionale Methode zur Auflistung aller Mitglieder
        public void ListAllMembers()
        {
            Console.WriteLine("Alle Mitglieder:");
            foreach (var member in members)
            {
                Console.WriteLine($"- {member.Name} (ID: {member.MemberID})");
            }
        }

        // Optionale Methode zur Auflistung aller Ausleihen
        public void ListAllLoans()
        {
            Console.WriteLine("Alle Ausleihen:");
            foreach (var loan in loans)
            {
                string status = loan.IsOverdue() ? "Überfällig" : "In Ordnung";
                Console.WriteLine($"- {loan.Member.Name} hat '{loan.Book.Title}' ausgeliehen. Fällig am: {loan.DueDate.ToShortDateString()} ({status})");
            }
        }
    }
}

